@@ -33,9 +33,6 @@
 
 */
 
-if (typeof (module) === "undefined" || !module.exports)
-  throw new Error("No exports");
-
 (function (exports) {
 
   // Enable Strict Mode
@@ -48,7 +45,7 @@ if (typeof (module) === "undefined" || !module.exports)
     SieveSaslLoginResponse,
     SieveSaslCramMd5Response,
     SieveGetScriptResponse,
-    SieveSaslScramSha1Response
+    SieveSaslScramShaResponse
   } = require("./SieveResponse.js");
 
   const { SieveCrypto } = require("./SieveCrypto.js");
@@ -88,9 +85,9 @@ if (typeof (module) === "undefined" || !module.exports)
    * Add a response listener to this request. The response listener will
    * be triggered in case a successful server response was received
    *
-   * @param {function} listener
+   * @param {Function} listener
    *   the listener which should be invoked when the server response
-   * @returns {void}
+   *
    */
   SieveAbstractRequest.prototype.addResponseListener
     = function (listener) {
@@ -106,7 +103,7 @@ if (typeof (module) === "undefined" || !module.exports)
    * init request upon connecting or after tls completed.
    * Both are send by the server to the client.
    *
-   * @returns {Boolean}
+   * @returns {boolean}
    *   true in case the request is unsolicited. Which means
    *   the client sends a request and the server responds
    *   to that.
@@ -131,7 +128,7 @@ if (typeof (module) === "undefined" || !module.exports)
    * @param  {SieveAbstractRequestBuilder} builder
    *   a reference to a stateless request builder which can be used
    *   to form the request string.
-   * @return {String}
+   * @returns {string}
    *   the data which should be send to the server
    *
    * @abstract
@@ -154,7 +151,7 @@ if (typeof (module) === "undefined" || !module.exports)
 
   SieveAbstractRequest.prototype.onBye
     = function (response) {
-      if ((response.getResponse() === 1) && (this.byeListener))
+      if ((response.getResponse() === RESPONSE_BYE) && (this.byeListener))
         this.byeListener.onByeResponse(response);
     };
 
@@ -169,7 +166,7 @@ if (typeof (module) === "undefined" || !module.exports)
    *
    * @param {SieveSimpleResponse} response
    *   thr response which should be handled by this request.
-   * @returns {void}
+   *
    */
   SieveAbstractRequest.prototype.addResponse
     = function (response) {
@@ -206,9 +203,9 @@ if (typeof (module) === "undefined" || !module.exports)
   /**
    * Sets the sasl mechanisms username. Not all SASL Mechanisms require an username.
    *
-   * @param {String} username
+   * @param {string} username
    *   the username
-   * @return {void}
+   *
    **/
   SieveAbstractSaslRequest.prototype.setUsername
     = function (username) {
@@ -220,7 +217,7 @@ if (typeof (module) === "undefined" || !module.exports)
    * But there are also mechanisms like SASL EXTERNAL which does not need any passwords.
    * They use different methods to transfer the credentials.
    *
-   * @return {Boolean}
+   * @returns {boolean}
    *   indicates if this SASL Mechanism needs a password
    */
   SieveAbstractSaslRequest.prototype.hasPassword
@@ -231,7 +228,7 @@ if (typeof (module) === "undefined" || !module.exports)
   /**
    * Sets the sasl request's password.
    *
-   * @param {String} password
+   * @param {string} password
    *   the password which shall be used for the authentication.
    * @returns {SieveAbstractSaslRequest}
    *   a self reference
@@ -252,7 +249,7 @@ if (typeof (module) === "undefined" || !module.exports)
    * authorization which switch the user. Typically admins and superusers have
    * such super powers.
    *
-   * @returns {Boolean}
+   * @returns {boolean}
    *   true in case the request supports authorization otherwise false.
    */
   SieveAbstractSaslRequest.prototype.isAuthorizable
@@ -265,7 +262,7 @@ if (typeof (module) === "undefined" || !module.exports)
    * Sets the username which should be authorized.
    * In case authorization is not supported it will be silently ignored.
    *
-   * @param {String} authorization
+   * @param {string} authorization
    *   the username used for authorization
    * @returns {SieveAbstractRequest}
    *   a self reference
@@ -290,7 +287,7 @@ if (typeof (module) === "undefined" || !module.exports)
    * Loads a script from the server and returns the content.
    * In case the script is non existant an error will be triggered.
    *
-   * @param {String} script
+   * @param {string} script
    *   the script which should be retrived
    * @author Thomas Schmid
    * @constructor
@@ -315,11 +312,13 @@ if (typeof (module) === "undefined" || !module.exports)
         this.responseListener.onGetScriptResponse(response);
     };
 
-  /** @param {SieveResponseParser} parser */
+  /**
+   * @inheritdoc
+   */
   SieveGetScriptRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveGetScriptResponse(this.script, parser));
+        (new SieveGetScriptResponse(this.script)).parse(parser));
     };
 
   // ****************************************************************************//
@@ -331,9 +330,9 @@ if (typeof (module) === "undefined" || !module.exports)
    *
    * Please not it will overwrite silently any existing script with the same name.
    *
-   * @param {String} script
+   * @param {string} script
    *   the script's name
-   * @param {String} body
+   * @param {string} body
    *   the sieve script which should be stored on the server.
    *
    * @constructor
@@ -343,6 +342,8 @@ if (typeof (module) === "undefined" || !module.exports)
     this.script = script;
 
     // cleanup linebreaks...
+
+    // eslint-disable-next-line no-control-regex
     this.body = body.replace(/\r\n|\r|\n|\u0085|\u000C|\u2028|\u2029/g, "\r\n");
   }
 
@@ -364,11 +365,13 @@ if (typeof (module) === "undefined" || !module.exports)
         this.responseListener.onPutScriptResponse(response);
     };
 
-  /** @param {SieveResponseParser} parser */
+  /**
+   * @inheritdoc
+   */
   SievePutScriptRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveSimpleResponse(parser));
+        (new SieveSimpleResponse()).parse(parser));
     };
 
   // ****************************************************************************//
@@ -382,14 +385,13 @@ if (typeof (module) === "undefined" || !module.exports)
    *
    * An OK response can contain Syntax Warnings.
    *
-   * @example
    *   C: CheckScript {31+}
    *   C: #comment
    *   C: InvalidSieveCommand
    *   C:
    *   S: NO "line 2: Syntax error"
    *
-   * @param {String} body
+   * @param {string} body
    *   the script which should be check for syntactical validity
    *
    * @constructor
@@ -403,6 +405,8 @@ if (typeof (module) === "undefined" || !module.exports)
     // ... according to the documentation. But for some unknown reason a ...
     // ... string sometimes  contains mixed line breaks. Thus we convert ...
     // ... any \r\n, \r and \n to \r\n.
+
+    // eslint-disable-next-line no-control-regex
     this.body = body.replace(/\r\n|\r|\n|\u0085|\u000C|\u2028|\u2029/g, "\r\n");
     // this.body = UTF8Encode(body).replace(/\r\n|\r|\n/g, "\r\n");
   }
@@ -423,10 +427,13 @@ if (typeof (module) === "undefined" || !module.exports)
         this.responseListener.onCheckScriptResponse(response);
     };
 
+  /**
+   * @inheritdoc
+   */
   SieveCheckScriptRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveSimpleResponse(parser));
+        (new SieveSimpleResponse()).parse(parser));
     };
 
   // ****************************************************************************//
@@ -440,7 +447,7 @@ if (typeof (module) === "undefined" || !module.exports)
    * You activate a Script by calling SETACTIVE and the scriptname. At activation
    * the previous active Script will become inactive.
    *
-   * @param {String} script - The script name which should be activated. Passing
+   * @param {string} script - The script name which should be activated. Passing
    * an empty string deactivates the active script.
    *
    * @author Thomas Schmid
@@ -471,11 +478,13 @@ if (typeof (module) === "undefined" || !module.exports)
         this.responseListener.onSetActiveResponse(response);
     };
 
-  /** @param {SieveResponseParser} parser */
+  /**
+   * @inheritdoc
+   */
   SieveSetActiveRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveSimpleResponse(parser));
+        (new SieveSimpleResponse()).parse(parser));
     };
 
   // ****************************************************************************//
@@ -504,17 +513,19 @@ if (typeof (module) === "undefined" || !module.exports)
         this.responseListener.onCapabilitiesResponse(response);
     };
 
-  /** @param {SieveResponseParser} parser */
+  /**
+   * @inheritdoc
+   */
   SieveCapabilitiesRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveCapabilitiesResponse(parser));
+        (new SieveCapabilitiesResponse()).parse(parser));
     };
 
   // ****************************************************************************//
 
   /**
-   * @param {String} script
+   * @param {string} script
    * @author Thomas Schmid
      * @constructor
    */
@@ -540,11 +551,13 @@ if (typeof (module) === "undefined" || !module.exports)
         this.responseListener.onDeleteScriptResponse(response);
     };
 
-  /** @param {SieveResponseParser} parser */
+  /**
+   * @inheritdoc
+   */
   SieveDeleteScriptRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveSimpleResponse(parser));
+        (new SieveSimpleResponse()).parse(parser));
     };
 
   // ****************************************************************************//
@@ -565,7 +578,7 @@ if (typeof (module) === "undefined" || !module.exports)
   SieveNoopRequest.prototype = Object.create(SieveAbstractRequest.prototype);
   SieveNoopRequest.prototype.constructor = SieveNoopRequest;
 
-  /** @return {String} */
+  /** @returns {string} */
   SieveNoopRequest.prototype.getNextRequest
     = function (builder) {
       return builder
@@ -579,11 +592,13 @@ if (typeof (module) === "undefined" || !module.exports)
         this.responseListener.onNoopResponse(response);
     };
 
-  /** @param {SieveResponseParser} parser */
+  /**
+   * @inheritdoc
+   */
   SieveNoopRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveSimpleResponse(parser));
+        (new SieveSimpleResponse()).parse(parser));
     };
 
   // ****************************************************************************//
@@ -595,8 +610,8 @@ if (typeof (module) === "undefined" || !module.exports)
    *
    * Renaming the active script is allowed, the renamed script remains active.
    *
-   * @param {String} oldScript Name of the script, which should be renamed
-   * @param {String} newScript New name of the Script
+   * @param {string} oldScript Name of the script, which should be renamed
+   * @param {string} newScript New name of the Script
    *
    * @author Thomas Schmid
    * @constructor
@@ -624,11 +639,13 @@ if (typeof (module) === "undefined" || !module.exports)
         this.responseListener.onRenameScriptResponse(response);
     };
 
-  /** @param {SieveResponseParser} parser */
+  /**
+   * @inheritdoc
+   */
   SieveRenameScriptRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveSimpleResponse(parser));
+        (new SieveSimpleResponse()).parse(parser));
     };
 
   // ****************************************************************************//
@@ -659,11 +676,13 @@ if (typeof (module) === "undefined" || !module.exports)
         this.responseListener.onListScriptResponse(response);
     };
 
-  /** @param {SieveResponseParser} parser */
+  /**
+   * @inheritdoc
+   */
   SieveListScriptRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveListScriptResponse(parser));
+        new SieveListScriptResponse().parse(parser));
     };
 
   // ****************************************************************************//
@@ -692,13 +711,13 @@ if (typeof (module) === "undefined" || !module.exports)
     };
 
   /**
-   *  @param {SieveResponseParser} parser
-   *  @returns {void}
-  */
+   * @inheritdoc
+   */
   SieveStartTLSRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveSimpleResponse(parser));
+        (new SieveSimpleResponse()).parse(parser));
+
     };
 
   // ****************************************************************************//
@@ -761,11 +780,13 @@ if (typeof (module) === "undefined" || !module.exports)
       this.onOk(response);
     };
 
-  /** @param {SieveResponseParser} parser */
+  /**
+   * @inheritdoc
+   */
   SieveLogoutRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveSimpleResponse(parser));
+        (new SieveSimpleResponse()).parse(parser));
     };
 
   // ****************************************************************************//
@@ -786,9 +807,9 @@ if (typeof (module) === "undefined" || !module.exports)
    *        < OK
    *
    * @example
-   *  var sieve = new Sieve("example.com",2000,false,3)
+   *  let sieve = new Sieve('example.com',2000,false,3)
    *
-   *  var request = new SieveInitRequest();
+   *  let request = new SieveInitRequest();
    *  sieve.addRequest(request);
    *
    *  sieve.connect();
@@ -812,11 +833,13 @@ if (typeof (module) === "undefined" || !module.exports)
     return false;
   };
 
-  /** @param {SieveResponseParser} parser */
+  /**
+   * @inheritdoc
+   */
   SieveInitRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveCapabilitiesResponse(parser));
+        (new SieveCapabilitiesResponse()).parse(parser));
     };
 
   /**
@@ -874,11 +897,13 @@ if (typeof (module) === "undefined" || !module.exports)
         .addQuotedBase64(this._authorization + "\0" + this._username + "\0" + this._password);
     };
 
-
+  /**
+   * @inheritdoc
+   */
   SieveSaslPlainRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveSimpleResponse(parser));
+        (new SieveSimpleResponse()).parse(parser));
     };
 
 
@@ -1003,13 +1028,14 @@ if (typeof (module) === "undefined" || !module.exports)
       return true;
     };
 
-
-  /** @param {SieveResponseParser} parser */
+  /**
+   * @inheritdoc
+   */
   SieveSaslLoginRequest.prototype.addResponse
     = function (parser) {
-      this.response.add(parser);
+      this.response.parse(parser);
 
-      if (this.response.getState() !== 4)
+      if (this.hasNextRequest())
         return;
 
       SieveAbstractRequest.prototype.addResponse.call(this, this.response);
@@ -1087,13 +1113,14 @@ if (typeof (module) === "undefined" || !module.exports)
       return true;
     };
 
-
-
+  /**
+   * @inheritdoc
+   */
   SieveSaslCramMd5Request.prototype.addResponse
     = function (parser) {
-      this.response.add(parser);
+      this.response.parse(parser);
 
-      if (this.response.getState() !== 4)
+      if (this.hasNextRequest())
         return;
 
       SieveAbstractRequest.prototype.addResponse.call(this, this.response);
@@ -1108,7 +1135,7 @@ if (typeof (module) === "undefined" || !module.exports)
    * @constructor
    */
   function SieveAbstractSaslScramRequest() {
-    this.response = new SieveSaslScramSha1Response();
+    this.response = new SieveSaslScramShaResponse();
   }
 
   // Inherrit prototypes from SieveAbstractRequest...
@@ -1267,11 +1294,15 @@ if (typeof (module) === "undefined" || !module.exports)
       SieveAbstractSaslRequest.prototype.onOk.call(this, response);
     };
 
+  /**
+   * @inheritdoc
+   */
   SieveAbstractSaslScramRequest.prototype.addResponse
     = function (parser) {
-      this.response.add(parser);
 
-      if (this.response.getState() !== 4)
+      this.response.parse(parser);
+
+      if (this.hasNextRequest())
         return;
 
       SieveAbstractRequest.prototype.addResponse.call(this, this.response);
@@ -1360,7 +1391,7 @@ if (typeof (module) === "undefined" || !module.exports)
    * SASL External uses the TLS Cert for authentication.
    * Thus it does not rely upon any password, so this mehtod retuns always false.
    *
-   * @return {Boolean}
+   * @returns {boolean}
    *   returns always false
    */
   SieveSaslExternalRequest.prototype.hasPassword
@@ -1368,10 +1399,13 @@ if (typeof (module) === "undefined" || !module.exports)
       return false;
     };
 
+  /**
+   * @inheritdoc
+   */
   SieveSaslExternalRequest.prototype.addResponse
     = function (parser) {
       SieveAbstractRequest.prototype.addResponse.call(this,
-        new SieveSimpleResponse(parser));
+        (new SieveSimpleResponse()).parse(parser));
     };
 
   exports.SieveGetScriptRequest = SieveGetScriptRequest;
@@ -1394,5 +1428,3 @@ if (typeof (module) === "undefined" || !module.exports)
   exports.SieveSaslExternalRequest = SieveSaslExternalRequest;
 
 })(module.exports || this);
-
-
