@@ -12,128 +12,131 @@
 
 /* global window */
 
-"use strict";
-
 (function (exports) {
 
+  "use strict";
   /* global SieveDataTransfer */
 
-  function SieveDropHandler() {
-  }
+  /**
+   *
+   */
+  class SieveDropHandler {
 
-  SieveDropHandler.prototype._flavours = [];
-  SieveDropHandler.prototype._owner = null;
+    /**
+     *
+     */
+    constructor(flavours) {
 
-  SieveDropHandler.prototype.flavours
-    = function (flavours, append) {
-      if (typeof (flavours) === "undefined")
-        return this._flavours;
+      this._flavours = [];
 
-      if (append)
-        this._flavours.concat(flavours);
-      else
+      if (typeof (flavours) !== "undefined" && flavours !== null)
         this._flavours = [].concat(flavours);
 
-      return this;
-    };
+      this._owner = null;
+    }
 
-  SieveDropHandler.prototype.document
-    = function () {
+    /**
+     * Gets the flavour for this drop type
+     * @returns {string[]}
+     *   a string array containing all supported drop flavours.
+     */
+    flavours() {
+      return this._flavours;
+    }
+
+    document() {
       if (!this._owner)
         throw new Error("Owner for this Drop Handler");
 
       return this._owner.document();
-    };
+    }
 
-  SieveDropHandler.prototype.bind
-    = function (owner, sibling) {
+    bind(owner, sibling) {
       this._owner = owner;
       this._sibling = sibling;
-    };
+    }
 
-  /**
-   * The owner on thich the drag event occured
-   * A Widget
-   * @return {}
-   */
-  SieveDropHandler.prototype.owner
-    = function () {
+    /**
+     * @returns {SieveAbstractWidget}
+     *   The owner on thich the drag event occured
+     */
+    owner() {
       return this._owner;
-    };
+    }
 
-  /**
-   * The target/sibling, the element which consumes the drop
-   * A Sieve Element
-   * @return {}
-   */
-  SieveDropHandler.prototype.sibling
-    = function () {
+    /**
+     * The target/sibling, the element which consumes the drop
+     * @returns {SieveAbstractElement}
+     *   the sibling element
+     */
+    sibling() {
       return this._sibling;
-    };
+    }
 
-  /**
-   * The parent of this element.
-   * A SieveAbstractElement
-   * @return {}
-   */
-  SieveDropHandler.prototype.parent
-    = function () {
+    /**
+     * The parent of this element.
+     * @returns {SieveAbstractElement}
+     *   the parent element
+     */
+    parent() {
       return this._owner.parent();
-    };
+    }
 
-  SieveDropHandler.prototype.attach
-    = function (html) {
+    /**
+     *
+     * @param {*} html
+     */
+    attach(html) {
 
       html
         .bind("drop", (e) => { return this.onDragDrop(e); })
         .bind("dragover", (e) => { return this.onDragOver(e); })
         .bind("dragleave", (e) => { return this.onDragExit(e); })
         .bind("dragenter", (e) => { return this.onDragEnter(e); });
-    };
+    }
 
-  /* Official HTML5 Drag&Drop events... */
+    /* Official HTML5 Drag&Drop events... */
+
+    onDragEnter(event) {
 
-  SieveDropHandler.prototype.onDragEnter
-    = function (event) {
       if (!this.canDrop(event))
         return true;
 
       this.owner().html().attr("sivDragging", "true");
 
       return false;
-    };
+    }
+
+    onDragExit(event) {
 
-  SieveDropHandler.prototype.onDragExit
-    = function (event) {
       this.owner().html().removeAttr("sivDragging");
 
       // Exit is only used for UI cleanup, so we should never cancel this event.
       // Our parent might want to do cleanup too.
       return true;
-    };
+    }
+
+    onDragOver(event) {
 
-  SieveDropHandler.prototype.onDragOver
-    = function (event) {
       if (!this.canDrop(event))
         return true;
 
       this.owner().html().attr("sivDragging", "true");
 
       return false;
-    };
+    }
+
+    onDragDrop(event) {
 
-  SieveDropHandler.prototype.onDragDrop
-    = function (event) {
       this.owner().html().removeAttr("sivDragging");
 
       if (!this.drop(event))
         return true;
 
       return false;
-    };
+    }
 
-  SieveDropHandler.prototype.onDrop
-    = function (flavour, event) {
+    onDrop(flavour, event) {
       let dt = new SieveDataTransfer(event.originalEvent.dataTransfer);
 
       let meta = JSON.parse(dt.getData(flavour));
@@ -162,14 +165,12 @@
 
           dt.clear();
           return true;
-
-        default:
-          throw new Error("Invalid action..." + meta.action);
       }
-    };
 
-  SieveDropHandler.prototype.drop
-    = function (event) {
+      throw new Error("Invalid action..." + meta.action);
+    }
+
+    drop(event) {
       for (let i = 0; i < this.flavours().length; i++) {
         if (!this.onCanDrop(this.flavours()[i], event))
           continue;
@@ -178,10 +179,9 @@
       }
 
       return true;
-    };
+    }
 
-  SieveDropHandler.prototype.onCanDrop
-    = function (flavour, event) {
+    onCanDrop(flavour, event) {
       let dt = new SieveDataTransfer(event.originalEvent.dataTransfer);
 
       let meta = dt.getData(flavour);
@@ -197,23 +197,36 @@
 
       switch (meta.action) {
         case "create":
-          if (!this.canCreateElement)
-            return false;
-
           return this.canCreateElement(flavour, meta.type);
 
         case "move":
-          if (!this.canMoveElement)
-            return false;
-
           return this.canMoveElement(flavour, meta.id);
       }
 
       return false;
-    };
+    }
 
-  SieveDropHandler.prototype.canDrop
-    = function (event) {
+    /**
+     *
+     * @param {*} sivFlavour
+     * @param {*} type
+     * @returns {boolean}
+     */
+    canCreateElement(sivFlavour, type) {
+      return false;
+    }
+
+    /**
+     *
+     * @param {*} sivFlavour
+     * @param {*} id
+     * @returns {boolean}
+     */
+    canMoveElement(sivFlavour, id) {
+      return false;
+    }
+
+    canDrop(event) {
       for (let i = 0; i < this.flavours().length; i++) {
         if (!this.onCanDrop(this.flavours()[i], event))
           continue;
@@ -224,34 +237,41 @@
       }
 
       return false;
-    };
-
-  //* ***************************************************************************//
-
-  function SieveBlockDropHandler() {
-    SieveDropHandler.call(this);
-    this.flavours(["sieve/action", "sieve/test", "sieve/operator"]);
+    }
   }
 
-  SieveBlockDropHandler.prototype = Object.create(SieveDropHandler.prototype);
-  SieveBlockDropHandler.prototype.constructor = SieveBlockDropHandler;
+  // ****************************************************************************//
 
-  SieveBlockDropHandler.prototype.canMoveElement
-    = function (sivFlavour, id) {
+  /**
+   *
+   */
+  class SieveBlockDropHandler extends SieveDropHandler {
+
+    /**
+     * @inheritdoc
+     */
+    constructor() {
+      super(
+        ["sieve/action", "sieve/test", "sieve/operator"]);
+    }
+
+
+    /**
+     * @inheritdoc
+     */
+    canMoveElement(sivFlavour, id) {
       let source = this.document().id(id);
 
-      if (source.html().parent().prev().get(0) === this.owner().html().get(0))
+      if (source.html().parent().prev().get(0) == this.owner().html().get(0))
         return false;
 
-      if (source.html().parent().next().get(0) === this.owner().html().get(0))
+      if (source.html().parent().next().get(0) == this.owner().html().get(0))
         return false;
 
       return true;
+    }
 
-    };
-
-  SieveBlockDropHandler.prototype.moveOperator
-    = function (source, target) {
+    moveTest(source, target) {
 
       // Create a new Condition...
       let newCondition = this.document().createByName("condition");
@@ -284,10 +304,10 @@
       source.widget().reflow();
       oldOwner.widget().reflow();
 
-    };
+    }
+
+    moveAction(source, target) {
 
-  SieveBlockDropHandler.prototype.moveAction
-    = function (source, target) {
       // remember owner
       let oldOwner = source.remove(true, target);
       // Move Item to new owner
@@ -296,42 +316,42 @@
       // refresh old and new Owner
       target.widget().reflow();
       oldOwner.widget().reflow();
-    };
+    }
 
-  SieveBlockDropHandler.prototype.moveElement
-    = function (sivFlavour, id) {
-      let dragElm = this.document().id(id);
-      if (!dragElm)
+    moveElement(sivFlavour, id) {
+      let source = this.document().id(id);
+      if (!source)
         throw new Error("Block Drop Handler: No Element found for " + id);
 
-      let item = this.parent().getSieve();
-      if (!item)
+      let target = this.parent().getSieve();
+      if (!target)
         throw new Error("Block Drop Handler: No Element found for " + this.parent().id());
 
       switch (sivFlavour) {
         case "sieve/test":
         case "sieve/operator":
-          this.moveOperator(dragElm, item);
+          this.moveTest(source, target);
           return;
 
         case "sieve/action":
-          this.moveAction(dragElm, item);
+          this.moveAction(source, target);
           return;
       }
 
       throw new Error("Incompatible Drop");
-    };
+    }
 
-  SieveBlockDropHandler.prototype.canCreateElement
-    = function (sivFlavour, type) {
+    /**
+     * @inheritdoc
+     */
+    canCreateElement(sivFlavour, type) {
       if (sivFlavour === "sieve/operator")
         return false;
 
       return true;
-    };
+    }
 
-  SieveBlockDropHandler.prototype.createElement
-    = function (sivFlavour, type) {
+    createElement(sivFlavour, type) {
 
       let item = this.parent().getSieve();
 
@@ -340,33 +360,45 @@
 
       let elm = null;
 
-      if (sivFlavour === "sieve/test")
+      if (sivFlavour === "sieve/test") {
         elm = item.document().createByName("condition",
           "if " + item.document().createByName(type).toScript() + "{\r\n}\r\n");
-      else
+        item.append(elm, this.sibling());
+      }
+      else if (sivFlavour === "sieve/action") {
         elm = item.document().createByName(type);
 
-      item.append(elm, this.sibling());
+        item.append(elm, this.sibling());
+        // item.append( item.document().createByName( "whitespace", "\r\n" ), this.sibling() );
+      }
+      else {
+        throw new Error("Unknown Element " + type);
+      }
+
       item.widget().reflow();
-    };
-
-  //* ***************************************************************************//
-
-  function SieveTrashBoxDropHandler() {
-    SieveDropHandler.call(this);
-    this.flavours(["sieve/action", "sieve/test", "sieve/if", "sieve/operator"]);
+    }
   }
 
-  SieveTrashBoxDropHandler.prototype = Object.create(SieveDropHandler.prototype);
-  SieveTrashBoxDropHandler.prototype.constructor = SieveTrashBoxDropHandler;
+  /**
+   *
+   */
+  class SieveTrashBoxDropHandler extends SieveDropHandler {
 
-  SieveTrashBoxDropHandler.prototype.canMoveElement
-    = function (sivFlavour, id) {
+    /**
+     * @inheritdoc
+     */
+    constructor() {
+      super(["sieve/action", "sieve/test", "sieve/if", "sieve/operator"]);
+    }
+
+    /**
+     * @inheritdoc
+     */
+    canMoveElement(sivFlavour, id) {
       return true;
-    };
+    }
 
-  SieveTrashBoxDropHandler.prototype.moveElement
-    = function (sivFlavour, id) {
+    moveElement(sivFlavour, id) {
 
       let item = this.document().id(id);
       if (!item)
@@ -380,24 +412,27 @@
       item.widget().reflow();
 
       window.setTimeout(() => { this.document().compact(); }, 0);
-    };
+    }
+  }
 
-  //* ***************************************************************************//
+  // ****************************************************************************//
 
   /**
    * Implements an handler for Sieve Test actions..
-   * @constructor
    */
-  function SieveConditionDropHandler() {
-    SieveDropHandler.call(this);
-    this.flavours(["sieve/test", "sieve/action", "sieve/operator"]);
-  }
+  class SieveConditionDropHandler extends SieveDropHandler {
 
-  SieveConditionDropHandler.prototype = Object.create(SieveDropHandler.prototype);
-  SieveConditionDropHandler.prototype.constructor = SieveConditionDropHandler;
+    /**
+     * @inheritdoc
+     */
+    constructor() {
+      super(["sieve/test", "sieve/action", "sieve/operator"]);
+    }
 
-  SieveConditionDropHandler.prototype.canMoveElement
-    = function (flavour, id) {
+    /**
+     * @inheritdoc
+     */
+    canMoveElement(flavour, id) {
 
       // actions can only be added as last element...
       if (flavour === "sieve/action")
@@ -445,10 +480,9 @@
         return false;
 
       return true;
-    };
+    }
 
-  SieveConditionDropHandler.prototype.moveElement
-    = function (flavour, id) {
+    moveElement(flavour, id) {
       let oldOwner;
 
       let source = this.document().id(id);
@@ -502,10 +536,12 @@
       conditional.widget().reflow();
 
       return;
-    };
+    }
 
-  SieveConditionDropHandler.prototype.canCreateElement
-    = function (flavour, type) {
+    /**
+     * @inheritdoc
+     */
+    canCreateElement(flavour, type) {
       if (flavour === "sieve/operator")
         return false;
 
@@ -529,10 +565,9 @@
         return false;
 
       return true;
-    };
+    }
 
-  SieveConditionDropHandler.prototype.createElement
-    = function (sivFlavour, type) {
+    createElement(sivFlavour, type) {
       // The new home for our element
       let item = this.parent().getSieve();
       let elm = null;
@@ -564,20 +599,25 @@
       }
 
       throw new Error("Incompatible drop");
-    };
-
-  //* ***************************************************************************//
-
-  function SieveTestDropHandler() {
-    SieveDropHandler.call(this);
-    this.flavours(["sieve/operator", "sieve/test"]);
+    }
   }
 
-  SieveTestDropHandler.prototype = Object.create(SieveDropHandler.prototype);
-  SieveTestDropHandler.prototype.constructor = SieveTestDropHandler;
+  /**
+   *
+   */
+  class SieveTestDropHandler extends SieveDropHandler {
 
-  SieveTestDropHandler.prototype.canMoveElement
-    = function (sivFlavour, id) {
+    /**
+     * @inheritdoc
+     */
+    constructor() {
+      super(["sieve/operator", "sieve/test"]);
+    }
+
+    /**
+     * @inheritdoc
+     */
+    canMoveElement(sivFlavour, id) {
       let target = this.owner().getSieve();
       if (!target)
         return false;
@@ -604,10 +644,9 @@
       }
 
       return true;
-    };
+    }
 
-  SieveTestDropHandler.prototype.moveElement
-    = function (sivFlavour, id) {
+    moveElement(sivFlavour, id) {
       let source = this.document().id(id);
       if (!source)
         throw new Error("Test Drop Handler: No Element found for " + id);
@@ -664,15 +703,16 @@
       if (newConditional)
         newConditional.widget().reflow();
       oldOwner.widget().reflow();
-    };
+    }
 
-  SieveTestDropHandler.prototype.canCreateElement
-    = function (sivFlavour, type) {
+    /**
+     * @inheritdoc
+     */
+    canCreateElement() {
       return true;
-    };
+    }
 
-  SieveTestDropHandler.prototype.createElement
-    = function (sivFlavour, type) {
+    createElement(sivFlavour, type) {
       // The new home for our element
       let inner = this.owner().getSieve();
 
@@ -704,28 +744,34 @@
       outer.parent(container);
       inner.parent(outer);
 
-      if (sivFlavour !== null)
+      if (sivFlavour === "sieve/test")
         outer.append(test);
 
       // newOwner.wrap(item.document().createByName(type))
       // item.widget().reflow();
 
       container.widget().reflow();
-    };
-
-  //* ***************************************************************************//
-
-  // used in multary operators
-  function SieveMultaryDropHandler() {
-    SieveDropHandler.call(this);
-    this.flavours(["sieve/operator", "sieve/test"]);
+    }
   }
 
-  SieveMultaryDropHandler.prototype = Object.create(SieveDropHandler.prototype);
-  SieveMultaryDropHandler.prototype.constructor = SieveMultaryDropHandler;
 
-  SieveMultaryDropHandler.prototype.canMoveElement
-    = function (sivFlavour, id) {
+  // used in multary operators
+  /**
+   *
+   */
+  class SieveMultaryDropHandler extends SieveDropHandler {
+
+    /**
+     * @inheritdoc
+     */
+    constructor() {
+      super(["sieve/operator", "sieve/test"]);
+    }
+
+    /**
+     * @inheritdoc
+     */
+    canMoveElement(sivFlavour, id) {
       // We have to prevent that someone drops a parent onto a child...
       //  ... this would generate a ring reference
       let target = this.parent().getSieve();
@@ -745,17 +791,16 @@
       }
 
       // It makes no sense so drop the item directly before or after the element.
-      if (source.html().parent().prev().get(0) === this.owner().html().get(0))
+      if (source.html().parent().prev().get(0) == this.owner().html().get(0))
         return false;
 
-      if (source.html().parent().next().get(0) === this.owner().html().get(0))
+      if (source.html().parent().next().get(0) == this.owner().html().get(0))
         return false;
 
       return true;
-    };
+    }
 
-  SieveMultaryDropHandler.prototype.moveElement
-    = function (sivFlavour, id) {
+    moveElement(sivFlavour, id) {
       let target = this.parent().getSieve();
 
       if (!target)
@@ -798,18 +843,19 @@
         newConditional.widget().reflow();
       oldOwner.widget().reflow();
 
-    };
+    }
 
-  SieveMultaryDropHandler.prototype.canCreateElement
-    = function (sivFlavour, type) {
+    /**
+     * @inheritdoc
+     */
+    canCreateElement(sivFlavour, type) {
       if (sivFlavour !== "sieve/test")
         return false;
 
       return true;
-    };
+    }
 
-  SieveMultaryDropHandler.prototype.createElement
-    = function (sivFlavour, type) {
+    createElement(sivFlavour, type) {
       let item = this.parent().getSieve();
 
       if (!item)
@@ -819,7 +865,8 @@
 
       item.append(elm, this.sibling());
       item.widget().reflow();
-    };
+    }
+  }
 
   exports.SieveDropHandler = SieveDropHandler;
   exports.SieveMultaryDropHandler = SieveMultaryDropHandler;
